#include <windows.h>
#include <stdio.h>
#include "ScrollBar.h"
#include "resource.h"
#if defined (win32)
   #define IS_WIN32 TRUE
#else
   #define IS_WIN32 FALSE
#endif

#define RANGEMAX 50

char str[255]; /* holds output strings */

int X=0, Y=0; /* current output location */

HINSTANCE hInst;        // current instance
LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle =   "ScrollBar Inside Dialog Application";
BOOL RegisterWin95(CONST WNDCLASS* lpwc);
void ShowPos(HWND hDlg, int nPos);

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
 {
   MSG msg;
   HWND hWnd;
   WNDCLASS wc;

   wc.style          = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc    = (WNDPROC)WndProc;
   wc.cbClsExtra     = 0;
   wc.cbWndExtra     = 0;
   wc.hInstance      = 0;
   wc.hIcon          = LoadIcon(hInstance, lpszAppName);
   wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground  = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName   = lpszAppName;
   wc.lpszClassName  = lpszAppName;

   if(!RegisterWin95(&wc))
      return false;
   hInst = hInstance;
   hWnd = CreateWindow (lpszAppName,
                        lpszTitle,
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, 0,
                        CW_USEDEFAULT, 0,
                        NULL,
                        NULL,
                        hInstance,
                        NULL
                       );
   if(!hWnd)
      return false;
   ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);
   while(GetMessage(&msg, NULL, 0,0))
    {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
    }
   return(msg.wParam);
}

BOOL RegisterWin95(CONST WNDCLASS* lpwc)
 {
   WNDCLASSEX wcex;

   wcex.style           = lpwc->style;
   wcex.lpfnWndProc     = lpwc->lpfnWndProc;
   wcex.cbClsExtra      = lpwc->cbClsExtra;
   wcex.cbWndExtra      = lpwc->cbWndExtra;
   wcex.hInstance       = lpwc->hInstance;
   wcex.hIcon           = lpwc->hIcon;
   wcex.hCursor         = lpwc->hCursor;
   wcex.hbrBackground   = lpwc->hbrBackground;
   wcex.lpszMenuName    = lpwc->lpszMenuName;
   wcex.lpszClassName   = lpwc->lpszClassName;
   wcex.cbSize          = sizeof(WNDCLASSEX);
   wcex.hIconSm         = LoadIcon(wcex.hInstance, "SMALL");
   return RegisterClassEx(&wcex);
 }

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   switch(uMsg)
    {
      case WM_COMMAND:
         switch(LOWORD(wParam))
          {
            case IDM_TEST :
               
			   DialogBox( hInst, "TestDialog", hWnd,
                                   (DLGPROC)TestDlgProc );
			   
               break;
            case IDM_EXIT :
               DestroyWindow(hWnd);
               break;
          }
         break;
      case WM_DESTROY :
         PostQuitMessage(0);
         break;
      default:
         return (DefWindowProc(hWnd, uMsg, wParam, lParam));
    }
   return(0L);
}

LRESULT CALLBACK TestDlgProc( HWND hDlg, UINT uMsg, 
                              WPARAM wParam, LPARAM lParam )
{
  
   static int pos = 0; /* slider box position */
   
   switch( uMsg )
   {
   case WM_INITDIALOG:
      SetScrollRange((HWND) GetDlgItem(hDlg,IDD_SB1), SB_CTL, 0, RANGEMAX, 1);
	  break; 
   case WM_VSCROLL:  
      
      switch(LOWORD(wParam)) {
        case SB_LINEDOWN:
          pos++;
          if(pos>RANGEMAX) pos = RANGEMAX;
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);
          ShowPos(hDlg, pos);
		  break;
        case SB_LINEUP:
          pos--;
          if(pos<0) pos = 0;
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);
          ShowPos(hDlg, pos);
		  break;
        case SB_THUMBPOSITION:
          pos = HIWORD(wParam); /* get current position */
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);          
          ShowPos(hDlg, pos);
		  break;
        case SB_THUMBTRACK:
          pos = HIWORD(wParam); /* get current position */
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);
          ShowPos(hDlg, pos);
		  break;
        case SB_PAGEDOWN:
          pos += 5;
          if(pos>RANGEMAX) pos = RANGEMAX;
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);
          ShowPos(hDlg, pos);
		  break;
        case SB_PAGEUP:
          pos -= 5;
          if(pos<0) pos = 0;
          SetScrollPos((HWND) lParam, SB_CTL, pos, 1);
		  ShowPos(hDlg, pos);
          break;
      }
      break;
      case WM_COMMAND :
            switch( LOWORD( wParam ) )
            {
               case IDOK: 
				    EndDialog( hDlg, IDOK );
                    break;
               case IDCANCEL : 
                    EndDialog( hDlg, IDCANCEL );
                    break;
			}
            break;

      default :
            return( FALSE );
   }

   return( TRUE );
}

void ShowPos(HWND hDlg, int nPos) /* Display the position  of the scroll bar. */
{
   HDC hDC;
   char str[80];


   hDC = GetDC(hDlg);

   sprintf(str, "%d", nPos);
   TextOut(hDC, 55, 30, "    ", 4);
   TextOut(hDC, 55, 30, str, strlen(str));
   
   ReleaseDC(hDlg, hDC);
}