#include "genstub.cpp"

CRITICAL_SECTION cs;

// Child thread procedure.
DWORD WINAPI ChildThreadProc(HWND hWnd)
{
   int Index;
   Index = ((int) GetCurrentThreadId() & 27) ;
   
   TCHAR szBuffer[256];    // Work area for print formatting.
   wsprintf( szBuffer, "Thread %x waiting for critical section", GetCurrentThreadId() );
   SendMessage( hWnd, WM_USER, WPARAM(Index), (LPARAM) szBuffer );
   // Enter protected code.
   EnterCriticalSection( &cs );
   wsprintf( szBuffer,"Thread %x in critical section", GetCurrentThreadId() );
   SendMessage( hWnd, WM_USER, WPARAM(Index), (LPARAM) szBuffer );
   Sleep( 5000 );  // This is where our critical code would go if we had some.
   // All done now, let someone else in.
   LeaveCriticalSection( &cs );
   wsprintf( szBuffer,"Thread %x has exited critical section", GetCurrentThreadId() );
   SendMessage( hWnd, WM_USER, WPARAM(Index), (LPARAM) szBuffer );
   ExitThread(TRUE);
   return ( TRUE );
}

// Main Window Procedure
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
   switch (uMsg)
   {
           case WM_CREATE:  // A critical section must be initialized before it is used.
                   InitializeCriticalSection(&cs);
                   return DefWindowProc( hWnd, uMsg, wParam, lParam );
           case WM_COMMAND:       // Process menu items.
                   switch ( LOWORD( wParam ) )
                   {
                        case IDM_TEST:
                        {
                           DWORD dwChildID;
                           CreateThread( NULL, 0, (LPTHREAD_START_ROUTINE)ChildThreadProc, hWnd, 0, &dwChildID );
                        }
                        break;
                        case IDM_EXIT:
                           DestroyWindow( hWnd );
                           break;
                   }
           break;
           case WM_USER:
                   {  // Show synchronization activity.
                      TCHAR szBuffer[101];
                      static int row = 0;
                      static int msg_num = 1;
                      HDC hDC = GetDC( hWnd );
                      FillMemory( szBuffer, 100, 32 );
					  switch (wParam%5)
					  {
					  case 0:
						  SetTextColor(hDC, RGB(0,255,255));
						  break;
					  case 1:
						  SetTextColor(hDC, RGB(255,0,0));
						  break;
					  case 2:
						  SetTextColor(hDC, RGB(0,0,255));
						  break;
					  case 3:
						  SetTextColor(hDC, RGB(0,0,0));
						  break;
					  case 4:
						  SetTextColor(hDC, RGB(0,255,0));
						  break;
					  
					  }

                      TextOut( hDC, 0, row, szBuffer, 100 );
                      wsprintf( szBuffer, "%3d: %s", msg_num++, (LPTSTR)lParam );
                      TextOut( hDC, 0, row, szBuffer, lstrlen( szBuffer ) );
                      if ( row > 200 )
                         row = 0;
                      else
                         row += 20;
                      ReleaseDC( hWnd, hDC );
                   }
                   break;
           case WM_DESTROY: // Get back resources from critical section object.
                   DeleteCriticalSection(&cs);
                   PostQuitMessage( 0 );
                   break;
           default:
                return DefWindowProc( hWnd, uMsg, wParam, lParam );
   }
   return NULL;
}